import hashlib
import os


def to_hash(string):
    """ Hash a string, in this case a url.

    :param str: a str containing an url
    :return str: a sha256 hashed string
    """
    hash_object = hashlib.sha256(bytes(string.encode("utf-8")))
    return hash_object.hexdigest()


class FSCache:
    """ File System KV cache for URLs.
    It exposes the following methods:

    add(url, content)

    get(url)

    delete(url)

    delete_all()
    """
    def __init__(self):
        self.cache_dir = "/tmp/doc-fzf"
        self.create_cache_dir()

    def create_cache_dir(self):
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

    def add(self, key, value):
        hashed_key = to_hash(key)
        with open("{0}/{1}.html".format(self.cache_dir, hashed_key), "w", encoding='utf8') as fd:
            # TODO: fix this
            value = value.replace("\\xe2\\x80\\x93", "-")
            fd.write("{0} {1}".format(key, value))

    def get(self, key):
        hashed_key = to_hash(key)
        if os.path.isfile("{0}/{1}.html".format(self.cache_dir, hashed_key)):
            return open("{0}/{1}.html".format(self.cache_dir, hashed_key), "r", encoding='utf8')
        else:
            return None

    def delete(self, key):
        hashed_key = to_hash(key)
        try:
            os.remove("{0}/{1}.html".format(self.cache_dir, hashed_key))
        except OSError as error:
            print(error)

    def delete_all(self):
        files = [f for f in os.listdir(self.cache_dir) if f.endswith(".html")]
        for f in files:
            try:
                os.remove(os.path.join(self.cache_dir, f))
            except OSError as error:
                print(error)



#!/usr/bin/env python3
import argparse
import inspect
import os
import re
import sys
import webbrowser
from importlib import import_module

from iterfzf import iterfzf


def main():
    parser = argparse.ArgumentParser(description='doc-fzf.')
    parser.add_argument('module_name',
                        type=str,
                        help='Name of the module to search')
    parser.add_argument('-q',
                        dest="query",
                        type=str,
                        default=None,
                        help='Query the docs')
    args = parser.parse_args()

    # Dynamically load a module
    try:
        # I need to get installation folder and add them there
        print(os.path.dirname(os.path.abspath(__file__)))
        module_path = os.path.dirname(os.path.abspath(__file__))
        # print(sys.path)
        # if module_path not in sys.path:
        module_path = "{0}/modules".format(module_path)
        # print(module_path)
        sys.path.insert(0, module_path)
        # print(sys.path)
        runtime_module = import_module(args.module_name)
    except Exception as error:
        print("Error loading module {0}. Does it exist?".format(args.module_name))
        print(error)
        sys.exit(2)

    # Initializing dynamic module
    try:
        runtime_module = runtime_module.Scrapper()
    except Exception as error:
        print("Module cannot load Scrapper class, is it defined?")
        print(error)
        sys.exit(2)

    # To get a clean list to be displayed by FZF we want to display only the
    # description of the documentation but we want to preserve the URL in memory
    # to return it to our web browser.
    urls = []
    descriptions = []
    try:
        for doc in runtime_module.get_documentation():
            urls.append(doc[0])
            descriptions.append(doc[1])
    except Exception as error:
        raise(error)
        sys.exit(2)

    if args.query:
        # Allow the user to start FZF with a query
        url = iterfzf(descriptions, exact=True, query=args.query)
    else:
        # Otherwise, just give all the results
        url = iterfzf(descriptions, exact=True)

    # Is there a better way to do this?
    for u, d in zip(urls, descriptions):
        if url == d:
            try:
                # Open URL in a new tab
                webbrowser.open(u, new=2)
            except Exception as error:
                # If webbrowser cannot open the URL just print it.
                print(u)


if __name__ == "__main__":
    main()



"""
This module contains the base class from which modules will inhertit
"""

import os

import requests

from .cache import FSCache


class FZFDoc:
    """ Base class for doc-fzf modules """

    def __init__(self, documentation_url):
        self.documentation_url = documentation_url

    def load_html(self):
        """ Return an HTML string from a URL.

        HTML will be retreived from cache, if not found, it will fetch the page
        with a GET request, put it in cache and return the content
        """
        cache = FSCache()
        cached_content = cache.get(self.documentation_url)

        if cached_content:
            return cached_content
        else:
            page = requests.get(self.documentation_url)
            cache.add(self.documentation_url, str(page.content))
            return page.content

    def get_documentation(self):
        """ All module instances must implement this function.

        This function allows you to scrap your documentation website in any way you like,
        but always returning a tuple of (url, description)
        """
        raise NotImplementedError(
            "get_documentation() not implemented for this module")



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

install_requires = [
    'requests',
    'bs4',
    'iterfzf'
]

setup(
    name = "doc-fzf",
    version = "0.0.2",
    author = "Guillermo Garcia",
    author_email = "memogarcia@protonmail.com",
    description = ("Fuzzy Search documentation from the CLI"),
    license = "LICENSE",
    keywords = "documentation fzf fuzzy search",
    url = "https://gitlab.com/memogarcia/doc-fzf",
    packages=['doc_fzf'],
    entry_points={
        'console_scripts': [
            'doc-fzf = doc_fzf.doc_fzf:main',
        ],
    },
    install_requires=install_requires,
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)